@@ -1,19 +1,8 @@
 /*
-    Copyright 2004-2005 Andi Peredri <andi@ukr.net>
-    Copyright 2007-2008 Fela Winkelmolen <fela.kde@gmail.com>
+    SPDX-FileCopyrightText: 2004-2005 Andi Peredri <andi@ukr.net>
+    SPDX-FileCopyrightText: 2007-2008 Fela Winkelmolen <fela.kde@gmail.com>
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "abstractgrid.h"
@@ -21,6 +10,7 @@
 #include <QMap>
 #include <QString>
 #include <QDebug>
+#include <QRandomGenerator>
 
 AbstractCell::AbstractCell(int index)
     : m_index(index)
@@ -164,10 +154,11 @@ void AbstractGrid::initializeGrid(uint width, uint height, Wrapping wrapping)
     // select a random cell that is not yet shuffled
     // rotate such that initial and final states are not same
     // repeat above two steps until minimum moves equal to shuffle limit
+    auto *generator = QRandomGenerator::global();
     while(m_minimumMoves < shuffleLimit)
     {
         // selecting a random index
-        int index = qrand() % notShuffledCells.count();
+        int index = generator->bounded(notShuffledCells.count());
         int cellNo = notShuffledCells[index];
         // removing the selected index so that it must not be used again
         notShuffledCells.removeAt(index);
@@ -186,10 +177,10 @@ void AbstractGrid::initializeGrid(uint width, uint height, Wrapping wrapping)
         }
         // for every other case rotate 1..3 times
         else {
-            int rotation = qrand() % 3 + 1; // 1..3
+            int rotation = generator->bounded(3) + 1; // 1..3
             // cant rotate twice when m_minimumMoves == shuffleLimit - 1
             if (m_minimumMoves == shuffleLimit - 1 && rotation == 2){
-                rotation = (qrand() % 2)? 1 : 3; // 1 or 3
+                rotation = (generator->bounded(2))? 1 : 3; // 1 or 3
             }
             m_minimumMoves += (rotation == 3) ? 1 : rotation;
             while(rotation--) {
@@ -228,7 +219,8 @@ void AbstractGrid::print() {
 void AbstractGrid::createGrid()
 {
     // add a random server
-    server_index = qrand() % (cellCount());
+    auto *generator = QRandomGenerator::global();
+    server_index = generator->bounded(cellCount());
 
     // number of cells that aren't free
     int notFreeCells = 0;
@@ -243,15 +235,15 @@ void AbstractGrid::createGrid()
 
         QList<uint> list;
         list.append(server_index);
-        if (qrand() % 2) addRandomCable(list);
+        if (generator->bounded(2)) addRandomCable(list);
 
         // add some random cables...
         // the list empties if there aren't many free cells left
         // (because of addRandomCable() not doing anything)
         while (!list.isEmpty()) {
-            if (qrand() % 2) {
+            if (generator->bounded(2)) {
                 addRandomCable(list);
-                if (qrand() % 2) addRandomCable(list);
+                if (generator->bounded(2)) addRandomCable(list);
                 list.erase(list.begin());
             }
             else {
@@ -299,7 +291,7 @@ void AbstractGrid::addRandomCable(QList<uint>& list)
 
     QMap<Directions, int>::ConstIterator it = freeCells.constBegin();
     // move the iterator to a random direction connecting to a free cell
-    for (int i = qrand() % freeCells.count(); i > 0; --i) ++it;
+    for (int i = QRandomGenerator::global()->bounded(freeCells.count()); i > 0; --i) ++it;
 
     // add the cable in the direction of cell
     Directions newCables = Directions(m_cells[cell]->cables() | it.key());
@@ -371,7 +363,7 @@ int AbstractGrid::solutionCount()
 {
     MoveList possibleNextMoves;
     // TODO: put following in external function
-    foreach (AbstractCell *cell, m_cells) {
+    for (AbstractCell *cell : std::as_const(m_cells)) {
         if (!cell->hasBeenMoved()) {
             Directions dirs = cell->cables();
             Move move;
@@ -411,7 +403,7 @@ int AbstractGrid::solutionCount()
     // else
 
     int solutionsFound = 0;
-    foreach (const Move &nextMove, possibleNextMoves) {
+    for (const Move &nextMove : std::as_const(possibleNextMoves)) {
         int index = nextMove.index();
 
         switch (nextMove.move()) {
@@ -441,7 +433,7 @@ int AbstractGrid::solutionCount()
 bool AbstractGrid::movesDoneArePossible()
 {
 
-    foreach (AbstractCell *cell, m_cells) {
+    for (AbstractCell *cell : std::as_const(m_cells)) {
         if (!cell->hasBeenMoved()) continue;
 
         uint x = cell->index() % m_width;
@@ -493,7 +485,7 @@ bool AbstractGrid::movesDoneArePossible()
 
 bool AbstractGrid::hasUnneededCables()
 {
-    foreach (AbstractCell *cell, m_cells) {
+    for (AbstractCell *cell : std::as_const(m_cells)) {
         if (cell->isTerminal() || cell->isServer() || cell->cables() == None) {
             continue;
         }
@@ -515,7 +507,7 @@ bool AbstractGrid::hasUnneededCables()
 
 bool AbstractGrid::isPossibleSolution()
 {
-    foreach (AbstractCell *cell, m_cells) {
+    for (AbstractCell *cell : std::as_const(m_cells)) {
         cell->setConnected(false);
     }
     updateConnections();
@@ -525,7 +517,7 @@ bool AbstractGrid::isPossibleSolution()
 
 bool AbstractGrid::allTerminalsConnected() {
     // return false if there is a terminal that isn't connected
-    foreach (AbstractCell *cell, m_cells) {
+    for (AbstractCell *cell : std::as_const(m_cells)) {
         if (cell->isTerminal() && !cell->isConnected()) {
             return false;
         }
@@ -556,27 +548,27 @@ QList<int> AbstractGrid::updateConnections()
         int lindex = lCell(cell_index);
 
         AbstractCell *cell = m_cells[cell_index];
-        AbstractCell *ucell = (uindex != NO_CELL) ? m_cells[uindex] : 0;
-        AbstractCell *rcell = (rindex != NO_CELL) ? m_cells[rindex] : 0;
-        AbstractCell *dcell = (dindex != NO_CELL) ? m_cells[dindex] : 0;
-        AbstractCell *lcell = (lindex != NO_CELL) ? m_cells[lindex] : 0;
+        AbstractCell *ucell = (uindex != NO_CELL) ? m_cells[uindex] : nullptr;
+        AbstractCell *rcell = (rindex != NO_CELL) ? m_cells[rindex] : nullptr;
+        AbstractCell *dcell = (dindex != NO_CELL) ? m_cells[dindex] : nullptr;
+        AbstractCell *lcell = (lindex != NO_CELL) ? m_cells[lindex] : nullptr;
 
-        if ((cell->cables() & Up) && ucell != 0 &&
+        if ((cell->cables() & Up) && ucell != nullptr &&
                 (ucell->cables() & Down) && !newConnections[uindex]) {
             newConnections[uindex] = true;
             changedCells.append(ucell->index());
         }
-        if ((cell->cables() & Right) && rcell != 0 &&
+        if ((cell->cables() & Right) && rcell != nullptr &&
                 (rcell->cables() & Left) && !newConnections[rindex]) {
             newConnections[rindex] = true;
             changedCells.append(rcell->index());
         }
-        if ((cell->cables() & Down) && dcell != 0 &&
+        if ((cell->cables() & Down) && dcell != nullptr &&
                 (dcell->cables() & Up) && !newConnections[dindex]) {
             newConnections[dindex] = true;
             changedCells.append(dcell->index());
         }
-        if ((cell->cables() & Left) && lcell != 0 &&
+        if ((cell->cables() & Left) && lcell != nullptr &&
                 (lcell->cables() & Right) && !newConnections[lindex]) {
             newConnections[lindex] = true;
             changedCells.append(lcell->index());
