@@ -1,37 +1,28 @@
 /*
-    Copyright (C) 2013 by Ashwin Rajeev<ashwin_rajeev@hotmail.com>
+    SPDX-FileCopyrightText: 2013 Ashwin Rajeev <ashwin_rajeev@hotmail.com>
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #ifndef GAMEVIEW_H
 #define GAMEVIEW_H
 
-#include <KgDeclarativeView>
-#include <QVariant>
-#include <KgThemeProvider>
-#include <QQuickItem>
-
+// game
 #include "abstractgrid.h"
+// KDEGames
+#include <KgThemeProvider>
+// Qt
+#include <QQuickWidget>
+#include <QVariant>
+#include <QQuickItem>
 
 class KgSound;
 
-class GameView : public KgDeclarativeView
+class GameView : public QQuickWidget
 {
     Q_OBJECT
 public:
-    explicit GameView(QWidget *parent=0);
+    explicit GameView(QWidget *parent=nullptr);
     ~GameView() override;
     void startNewGame(uint width, uint height, Wrapping w);
     int minimumMoves() {return grid->minimumMoves();}
